=begin
(c) TIG 2011
Script:
TIG-exportDCs2csv.rb
Type:
  TIG.exportDCs2csv
in the Ruby Console to run it.
Exports all DCs is the model with a Name,LenX,LenY,LenZ,Layer[s]... 
'CSV' file - in the model's folder and named after the model thus:
ModelName.skp >>> ModelNameDCs.csv
If a new model is unsaved the current directory receives the new file.
All Layers used in the DC are listed by name and visible ones marked 
thus >>LayerName<<
Edit sep="," if something other than separating comma is desired e.g. ';'
Make sep="\t" if a TSV file is desired and change ext="csv" to ext="tsv".
It uses the current Model Units.
Version:
1.0 20111104 First issue.
1.1 20111221 DCs customized.
=end
require 'sketchup.rb'
###
module TIG

  def self.exportDCs2csv()
    sep="," ### <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
    ext="csv" ### <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
    model=Sketchup.active_model
    layers=model.layers
    defs=model.definitions
    dcs=[]
    defs.each{|dc|
      next if not dc.attribute_dictionary("dynamic_attributes")
      dcs << dc
    }
    if not dcs
      UI.messagebox("No DCs were available.\nExiting.")
	  return nil
    end#if
    insts=[]
    dcs.each{|dc|insts << dc.instances}
    insts.flatten!
    insts.uniq!
    insts.compact!
    data=[]
    insts.each{|inst|
      defn=inst.definition
      name=defn.name.tr(sep,"_")
      lenx=defn.get_attribute("dynamic_attributes","lenx",0).to_l.to_s
      leny=defn.get_attribute("dynamic_attributes","leny",0).to_l.to_s
      lenz=defn.get_attribute("dynamic_attributes","lenz",0).to_l.to_s
      dsp=defn.get_attribute("dynamic_attributes","dsp",0).to_l.to_s
      kromka=defn.get_attribute("dynamic_attributes","kromka",0).to_l.to_s
      dat=name+sep+lenx+sep+leny+sep+lenz+sep+dsp+sep+kromka
      dlayers=[]
      defn.entities.each{|e|dlayers << e.layer.name.tr(sep,"_") if not dlayers.include?(e.layer.name.tr(sep,"_"))}
      nlayers=[]
      dlayers.each{|ln|
        lnx=ln.gsub(/[^A-Za-z0-9_#{sep}]/,'_') ### trap '-/+/,' etc
        lnx=">>"+lnx+"<<" if layers[ln].visible?
        nlayers << lnx
      }
      nlayers.sort!
      nlayers.each{|lnx|dat=dat+sep+lnx}
      data << dat
    }
    path=model.path
    if not path or path==""
      path=Dir.pwd
      title="Untitled"
    else
      path=File.dirname(path)
      title=model.title
    end#if
    ofile=File.join(path,title+'DCs.'+ext).tr("\\","/")
    begin
      file=File.new(ofile,"w")
    rescue### trap if open
      UI.messagebox("File:\n\n  "+ofile+"\n\nCannot be written - it's probably already open.\nClose it and try making it again...\n\nExiting...")
	  return nil
    end
    data.each{|d|file.puts(d)}
    file.close
    puts (data.length.to_s)+" DCs written to\n"+ofile
    puts 'DONE.'
    begin
      UI.openURL("file:/"+ofile)
    rescue
    end
  end#def
  
end#module
###


